# -*- coding: cp1251 -*-

"""
Tool Name:     Check Digital Model
Source Name:   CheckDigitalModel.py
Version:       ArcGIS 10.x
Author:        Anatoly Baranov
Organization:  VSEGEI
Description:   Checks input directory and finds files containing no data
               (empty files) and folders containing no data or empty files.
               Creates a report with with an empty data list
"""

##################################### I N P U T S #####################################


import arcpy
from arcpy import env
from datetime import datetime


headFolder = arcpy.GetParameterAsText(0)  #  
createReport = arcpy.GetParameterAsText(1)  #  /     
logs = []  #     
emptyFolders = []  #     
emptyFiles = []  #     
if headFolder[0] == "'" and headFolder[-1] == "'":
    headFolder = headFolder[1:-1]


##################################### F U N C T I O N S #####################################


def createStructure(folder):
    """
        'folder' 
        , -  dbf-   JSON (dict)
                 :
            - <   > : { <     > },
            - 'files' : [ < -  dbf-> ],
     .
      'files'   'findEmpty()',
               . 
     : 'folder' -    
    """

    env.workspace = folder
    folders = [fold for fold in arcpy.ListWorkspaces() if not fold.endswith('.xls') and not fold.endswith('.xlsx')]

    usualFiles = [name for name in arcpy.ListFiles() if not (folder + '\\' + name in folders)]  #     
    specialFiles = []  #    

    #   -  dbf-
    tables = arcpy.ListFeatureClasses() + arcpy.ListTables()
    specialFiles.extend(tables)
    tables = [folder + '\\' + name for name in tables]
    tables = findEmpty(tables)

    #   
    rasters = arcpy.ListRasters()
    specialFiles.extend(rasters)
    if rasters: rasters = 'none'

    #     ,  ,   dbf-
    files = getFilesList(usualFiles, specialFiles)
    if files: files = 'none'

    struct = {'tables': tables, 'rasters': rasters, 'files': files}

    for fold in folders:
        struct[fold] = createStructure(fold)

    return struct


def getFilesList(usualFiles, specialFiles):
    resFiles = []

    number = len(usualFiles)
    usualBaseNames = [name.split('.')[0] for name in usualFiles]
    specialBaseNames = [name.split('.')[0] for name in specialFiles]

    for i in range(number):
        if not (usualBaseNames[i] in specialBaseNames):
            resFiles.append(usualFiles[i])

    return resFiles


def findEmpty(dataList):
    """
    ,  ,     - ,
          - ,        
        (<,    >, 'none', <  >, < >),
            

    datalist (list) -  
    """

    global logs

    if dataList:
        emptyList = []  #    
        for data in dataList: #   ,     'logs' (   )
            try:
                rowsNumber = int(arcpy.GetCount_management(data).getOutput(0))  # -  ()  
                if rowsNumber == 0:
                    emptyList.append(data)
            except:
                logs.append(data)
        #    
        if emptyList == dataList:
            res = tuple(dataList)
        #    
        elif not emptyList:
            res = 'none'
        #    ,      
        else:
            res = emptyList
    else:
        res = []  #    

    return res


def turnUpFolders(struct):
    """
          'struct'      ,
           ,   {}

    struct (dict) -  -
    """

    vals = [val for val in struct.values() if val and type(val) != tuple]
    if not vals:
        struct.clear()
    else:
        for val in vals:
            if type(val) == dict:
                turnUpFolders(val)


def getEmptyData(struct):

    global emptyFolders
    global emptyFiles

    keys = struct.keys()
    keys.sort()
    for key in keys:
        dat = struct[key]
        if type(dat) == dict:
            if dat == {}:
                emptyFolders.append(key)
            else:
                getEmptyData(dat)
        else:
            if dat != 'none':
                emptyFiles.extend(list(dat))


##################################### P R O C E S S I N G #####################################


#    
struct = createStructure(headFolder)

#     
dstr = str(struct)
dcopy = ''
while dcopy != dstr:
    dcopy = dstr
    turnUpFolders(struct)
    dstr = str(struct)
del dstr, dcopy

new_struct = {headFolder: struct}
del struct

#      
getEmptyData(new_struct)

#         
text = 'Results for CheckDigitalModel\n'
now = datetime.now().ctime()
text += now + '\n'
text += 'Source folder: ' + headFolder + '\n'
text += 80 * '-' + '\n'

#   
if emptyFolders:
    emptyFolders.sort()
    text += '\nEmpty folders:\n'
    for dat in emptyFolders:
        text += ' * ' + dat + '\n'

#   
if emptyFiles:
    emptyFiles.sort()
    text += '\nEmpty files:\n'
    for dat in emptyFiles:
        text += ' * ' + dat + '\n'

#     ,   ,     
if not emptyFolders and not emptyFiles:
    text += "Congratulations! Your folder contains no empty data\n"

#   ,   
if logs:
    logs.sort()
    text += '\nFailures:\n'
    for log in logs:
        text += ' * ' + log + '\n'

text += '\n' + 80 * '-'

#      
arcpy.AddMessage(text)

#     ,     
if createReport == 'true':
    filename = headFolder + '\\CheckDigitalModel_Report_' + now.replace(' ', '_').replace(':', '-') + '.txt'
    f = open(filename, 'w')
    f.write(text.encode('utf-8'))
    f.close()