# -*- coding: cp1251 -*-


"""
Tool Name:     Curly Bracket
Source Name:   CurlyBracket.py
Version:       ArcGIS 10.x
Author:        Anatoly Baranov
Organization:  VSEGEI
Description:   Creates a simple curly bracket (as polyline)
    and records it to the shapefile
"""


import arcpy
import math

orientation = arcpy.GetParameterAsText(0)
length = arcpy.GetParameterAsText(1)
point = arcpy.GetParameterAsText(2)
outShp = arcpy.GetParameterAsText(3)

geom = arcpy.Describe(outShp).shapeType # geometry shape type of output shapefile

if geom == 'Polyline':
    length = float(length.replace(',', '.')) # length of the bracket

    x, y = point.split(' ')
    x = float(x.replace(',', '.')) # start X coordinate
    y = float(y.replace(',', '.')) # start Y coordinate

    step = 6 # degrees in one step to make arc
    c = .1 # arc radius ( == hypotenuse )
    strSegm = length / 2 - 2 * c # bracket's straight segment length

    array = arcpy.Array()


    def arc(angleStart, angleEnd, xStart=0, yStart=0):
        '''
        Returns arcpy array object completed with
        X/Y coordinates of nodes of one arc
        '''
        
        point = arcpy.Point()
        array = arcpy.Array()

        # d - runway direction
        if angleEnd > angleStart: d = 1 # clockwise
        else: d = -1 # anticlockwise
        
        for angle in range(angleStart, angleEnd + step*d, step*d):
            rad = math.radians(angle)
            y = math.cos(rad) * c
            x = math.sin(rad) * c
            
            point.X = x + xStart
            point.Y = y + yStart
            array.add(point)
        
        return array



    if "HORIZONTAL" in orientation:
        if "UP" in orientation:
            x += c
            
            array.extend(arc(270, 360, x, y))
            x += strSegm; y += c*2

            array.extend(arc(180, 90, x, y))
            x += c*2

            array.extend(arc(270, 180, x, y))
            x += strSegm; y -= c*2

            array.extend(arc(0, 90, x, y))
        else:
            x += c
            
            array.extend(arc(270, 180, x, y))
            x += strSegm; y -= c*2

            array.extend(arc(0, 90, x, y))
            x += c*2

            array.extend(arc(270, 360, x, y))
            x += strSegm; y += c*2

            array.extend(arc(180, 90, x, y))
        
    else:
        if "LEFT" in orientation:
            y += c
            
            array.extend(arc(180, 270, x, y))
            y += strSegm; x -= c*2

            array.extend(arc(90, 0, x, y))
            y += c*2

            array.extend(arc(180, 90, x, y))
            y += strSegm; x += c*2

            array.extend(arc(270, 360, x, y))
        else:
            y += c
            
            array.extend(arc(180, 90, x, y))
            y += strSegm; x += c*2

            array.extend(arc(270, 360, x, y))
            y += c*2

            array.extend(arc(180, 270, x, y))
            y += strSegm; x -= c*2

            array.extend(arc(90, 0, x, y))


    polyline = arcpy.Polyline(array)
    array.removeAll()

    cur = arcpy.InsertCursor(outShp)
    row = cur.newRow()
    row.shape = polyline
    cur.insertRow(row)

    del cur
    del row
else:
    arcpy.AddMessage('\nThe geometry shape type of output feature class must be "Polyline"!\n')
